/* tslint:disable */
/* eslint-disable */
export const memory: WebAssembly.Memory;
export const __wbg_circuitbuilderwrapper_free: (a: number, b: number) => void;
export const __wbg_executionmodewrapper_free: (a: number, b: number) => void;
export const __wbg_iolayoutbuilderwrapper_free: (a: number, b: number) => void;
export const __wbg_iotypewrapper_free: (a: number, b: number) => void;
export const __wbg_layoutfunctionwrapper_free: (a: number, b: number) => void;
export const __wbg_mchprsworldwrapper_free: (a: number, b: number) => void;
export const __wbg_outputconditionwrapper_free: (a: number, b: number) => void;
export const __wbg_simulationoptionswrapper_free: (a: number, b: number) => void;
export const __wbg_statemodeconstants_free: (a: number, b: number) => void;
export const __wbg_typedcircuitexecutorwrapper_free: (a: number, b: number) => void;
export const __wbg_valuewrapper_free: (a: number, b: number) => void;
export const circuitbuilderwrapper_build: (a: number) => [number, number, number];
export const circuitbuilderwrapper_buildValidated: (a: number) => [number, number, number];
export const circuitbuilderwrapper_fromInsign: (a: number) => [number, number, number];
export const circuitbuilderwrapper_inputCount: (a: number) => number;
export const circuitbuilderwrapper_inputNames: (a: number) => [number, number];
export const circuitbuilderwrapper_new: (a: number) => number;
export const circuitbuilderwrapper_outputCount: (a: number) => number;
export const circuitbuilderwrapper_outputNames: (a: number) => [number, number];
export const circuitbuilderwrapper_validate: (a: number) => [number, number];
export const circuitbuilderwrapper_withInput: (a: number, b: number, c: number, d: number, e: number, f: number) => [number, number, number];
export const circuitbuilderwrapper_withInputAuto: (a: number, b: number, c: number, d: number, e: number) => [number, number, number];
export const circuitbuilderwrapper_withOptions: (a: number, b: number) => number;
export const circuitbuilderwrapper_withOutput: (a: number, b: number, c: number, d: number, e: number, f: number) => [number, number, number];
export const circuitbuilderwrapper_withOutputAuto: (a: number, b: number, c: number, d: number, e: number) => [number, number, number];
export const circuitbuilderwrapper_withStateMode: (a: number, b: number, c: number) => [number, number, number];
export const executionmodewrapper_fixedTicks: (a: number) => number;
export const executionmodewrapper_untilChange: (a: number, b: number) => number;
export const executionmodewrapper_untilCondition: (a: number, b: number, c: number, d: number, e: number) => number;
export const executionmodewrapper_untilStable: (a: number, b: number) => number;
export const iolayoutbuilderwrapper_addInput: (a: number, b: number, c: number, d: number, e: number, f: number, g: number) => [number, number, number];
export const iolayoutbuilderwrapper_addInputAuto: (a: number, b: number, c: number, d: number, e: number, f: number) => [number, number, number];
export const iolayoutbuilderwrapper_addInputFromRegion: (a: number, b: number, c: number, d: number, e: number, f: number) => [number, number, number];
export const iolayoutbuilderwrapper_addInputFromRegionAuto: (a: number, b: number, c: number, d: number, e: number) => [number, number, number];
export const iolayoutbuilderwrapper_addInputRegion: (a: number, b: number, c: number, d: number, e: number, f: number, g: number) => [number, number, number];
export const iolayoutbuilderwrapper_addInputRegionAuto: (a: number, b: number, c: number, d: number, e: number, f: number) => [number, number, number];
export const iolayoutbuilderwrapper_addOutput: (a: number, b: number, c: number, d: number, e: number, f: number, g: number) => [number, number, number];
export const iolayoutbuilderwrapper_addOutputAuto: (a: number, b: number, c: number, d: number, e: number, f: number) => [number, number, number];
export const iolayoutbuilderwrapper_addOutputFromRegion: (a: number, b: number, c: number, d: number, e: number, f: number) => [number, number, number];
export const iolayoutbuilderwrapper_addOutputFromRegionAuto: (a: number, b: number, c: number, d: number, e: number) => [number, number, number];
export const iolayoutbuilderwrapper_addOutputRegion: (a: number, b: number, c: number, d: number, e: number, f: number, g: number) => [number, number, number];
export const iolayoutbuilderwrapper_addOutputRegionAuto: (a: number, b: number, c: number, d: number, e: number, f: number) => [number, number, number];
export const iolayoutbuilderwrapper_build: (a: number) => number;
export const iolayoutbuilderwrapper_new: () => number;
export const iolayoutwrapper_inputNames: (a: number) => [number, number];
export const iolayoutwrapper_outputNames: (a: number) => [number, number];
export const iotypewrapper_ascii: (a: number) => number;
export const iotypewrapper_boolean: () => number;
export const iotypewrapper_float32: () => number;
export const iotypewrapper_signedInt: (a: number) => number;
export const iotypewrapper_unsignedInt: (a: number) => number;
export const layoutfunctionwrapper_columnMajor: (a: number, b: number, c: number) => number;
export const layoutfunctionwrapper_custom: (a: number, b: number) => number;
export const layoutfunctionwrapper_oneToOne: () => number;
export const layoutfunctionwrapper_packed4: () => number;
export const layoutfunctionwrapper_rowMajor: (a: number, b: number, c: number) => number;
export const layoutfunctionwrapper_scanline: (a: number, b: number, c: number) => number;
export const mchprsworldwrapper_checkCustomIoChanges: (a: number) => void;
export const mchprsworldwrapper_clearCustomIoChanges: (a: number) => void;
export const mchprsworldwrapper_flush: (a: number) => void;
export const mchprsworldwrapper_getSignalStrength: (a: number, b: number, c: number, d: number) => number;
export const mchprsworldwrapper_get_lever_power: (a: number, b: number, c: number, d: number) => number;
export const mchprsworldwrapper_get_redstone_power: (a: number, b: number, c: number, d: number) => number;
export const mchprsworldwrapper_get_schematic: (a: number) => number;
export const mchprsworldwrapper_get_truth_table: (a: number) => any;
export const mchprsworldwrapper_into_schematic: (a: number) => number;
export const mchprsworldwrapper_is_lit: (a: number, b: number, c: number, d: number) => number;
export const mchprsworldwrapper_new: (a: number) => [number, number, number];
export const mchprsworldwrapper_on_use_block: (a: number, b: number, c: number, d: number) => void;
export const mchprsworldwrapper_peekCustomIoChanges: (a: number) => any;
export const mchprsworldwrapper_pollCustomIoChanges: (a: number) => any;
export const mchprsworldwrapper_setSignalStrength: (a: number, b: number, c: number, d: number, e: number) => void;
export const mchprsworldwrapper_sync_to_schematic: (a: number) => void;
export const mchprsworldwrapper_tick: (a: number, b: number) => void;
export const mchprsworldwrapper_with_options: (a: number, b: number) => [number, number, number];
export const outputconditionwrapper_bitwiseAnd: (a: number) => number;
export const outputconditionwrapper_equals: (a: number) => number;
export const outputconditionwrapper_greaterThan: (a: number) => number;
export const outputconditionwrapper_lessThan: (a: number) => number;
export const outputconditionwrapper_notEquals: (a: number) => number;
export const simulationoptionswrapper_addCustomIo: (a: number, b: number, c: number, d: number) => void;
export const simulationoptionswrapper_clearCustomIo: (a: number) => void;
export const simulationoptionswrapper_io_only: (a: number) => number;
export const simulationoptionswrapper_new: () => number;
export const simulationoptionswrapper_optimize: (a: number) => number;
export const simulationoptionswrapper_set_io_only: (a: number, b: number) => void;
export const simulationoptionswrapper_set_optimize: (a: number, b: number) => void;
export const statemodeconstants_manual: () => [number, number];
export const statemodeconstants_stateful: () => [number, number];
export const statemodeconstants_stateless: () => [number, number];
export const typedcircuitexecutorwrapper_execute: (a: number, b: any, c: number) => [number, number, number];
export const typedcircuitexecutorwrapper_flush: (a: number) => void;
export const typedcircuitexecutorwrapper_fromInsign: (a: number) => [number, number, number];
export const typedcircuitexecutorwrapper_fromInsignWithOptions: (a: number, b: number) => [number, number, number];
export const typedcircuitexecutorwrapper_fromLayout: (a: number, b: number) => [number, number, number];
export const typedcircuitexecutorwrapper_fromLayoutWithOptions: (a: number, b: number, c: number) => [number, number, number];
export const typedcircuitexecutorwrapper_getLayoutInfo: (a: number) => any;
export const typedcircuitexecutorwrapper_inputNames: (a: number) => [number, number];
export const typedcircuitexecutorwrapper_outputNames: (a: number) => [number, number];
export const typedcircuitexecutorwrapper_readOutput: (a: number, b: number, c: number) => [number, number, number];
export const typedcircuitexecutorwrapper_reset: (a: number) => [number, number];
export const typedcircuitexecutorwrapper_setInput: (a: number, b: number, c: number, d: number) => [number, number];
export const typedcircuitexecutorwrapper_setStateMode: (a: number, b: number, c: number) => [number, number];
export const typedcircuitexecutorwrapper_syncToSchematic: (a: number) => number;
export const typedcircuitexecutorwrapper_tick: (a: number, b: number) => void;
export const valuewrapper_fromBool: (a: number) => number;
export const valuewrapper_fromF32: (a: number) => number;
export const valuewrapper_fromI32: (a: number) => number;
export const valuewrapper_fromString: (a: number, b: number) => number;
export const valuewrapper_fromU32: (a: number) => number;
export const valuewrapper_toJs: (a: number) => any;
export const valuewrapper_typeName: (a: number) => [number, number];
export const schematicwrapper_create_simulation_world_with_options: (a: number, b: number) => [number, number, number];
export const schematicwrapper_create_simulation_world: (a: number) => [number, number, number];
export const __wbg_iolayoutwrapper_free: (a: number, b: number) => void;
export const __wbg_definitionregionwrapper_free: (a: number, b: number) => void;
export const definitionregionwrapper_addBounds: (a: number, b: number, c: number) => void;
export const definitionregionwrapper_addPoint: (a: number, b: number, c: number, d: number) => void;
export const definitionregionwrapper_boxCount: (a: number) => number;
export const definitionregionwrapper_center: (a: number) => any;
export const definitionregionwrapper_centerF32: (a: number) => any;
export const definitionregionwrapper_clone: (a: number) => number;
export const definitionregionwrapper_connectedComponents: (a: number) => number;
export const definitionregionwrapper_contains: (a: number, b: number, c: number, d: number) => number;
export const definitionregionwrapper_contract: (a: number, b: number) => void;
export const definitionregionwrapper_contracted: (a: number, b: number) => number;
export const definitionregionwrapper_dimensions: (a: number) => any;
export const definitionregionwrapper_expand: (a: number, b: number, c: number, d: number) => void;
export const definitionregionwrapper_expanded: (a: number, b: number, c: number, d: number) => number;
export const definitionregionwrapper_filterByBlock: (a: number, b: number, c: number, d: number) => number;
export const definitionregionwrapper_filterByProperties: (a: number, b: number, c: any) => [number, number, number];
export const definitionregionwrapper_fromBoundingBoxes: (a: any) => [number, number, number];
export const definitionregionwrapper_fromBounds: (a: number, b: number) => number;
export const definitionregionwrapper_fromPositions: (a: any) => [number, number, number];
export const definitionregionwrapper_getAllMetadata: (a: number) => any;
export const definitionregionwrapper_getBounds: (a: number) => any;
export const definitionregionwrapper_getBox: (a: number, b: number) => any;
export const definitionregionwrapper_getBoxes: (a: number) => any;
export const definitionregionwrapper_getMetadata: (a: number, b: number, c: number) => any;
export const definitionregionwrapper_intersect: (a: number, b: number) => void;
export const definitionregionwrapper_intersected: (a: number, b: number) => number;
export const definitionregionwrapper_intersectsBounds: (a: number, b: number, c: number, d: number, e: number, f: number, g: number) => number;
export const definitionregionwrapper_isContiguous: (a: number) => number;
export const definitionregionwrapper_isEmpty: (a: number) => number;
export const definitionregionwrapper_merge: (a: number, b: number) => void;
export const definitionregionwrapper_metadataKeys: (a: number) => any;
export const definitionregionwrapper_new: () => number;
export const definitionregionwrapper_positions: (a: number) => any;
export const definitionregionwrapper_positionsSorted: (a: number) => any;
export const definitionregionwrapper_setMetadata: (a: number, b: number, c: number, d: number, e: number) => number;
export const definitionregionwrapper_shift: (a: number, b: number, c: number, d: number) => void;
export const definitionregionwrapper_shifted: (a: number, b: number, c: number, d: number) => number;
export const definitionregionwrapper_simplify: (a: number) => void;
export const definitionregionwrapper_subtract: (a: number, b: number) => void;
export const definitionregionwrapper_subtracted: (a: number, b: number) => number;
export const definitionregionwrapper_union: (a: number, b: number) => number;
export const definitionregionwrapper_unionInto: (a: number, b: number) => void;
export const definitionregionwrapper_volume: (a: number) => number;
export const definitionregionwrapper_copy: (a: number) => number;
export const __wbg_blockposition_free: (a: number, b: number) => void;
export const __wbg_get_blockposition_x: (a: number) => number;
export const __wbg_get_blockposition_y: (a: number) => number;
export const __wbg_get_blockposition_z: (a: number) => number;
export const __wbg_set_blockposition_x: (a: number, b: number) => void;
export const __wbg_set_blockposition_y: (a: number, b: number) => void;
export const __wbg_set_blockposition_z: (a: number, b: number) => void;
export const blockposition_new: (a: number, b: number, c: number) => number;
export const __wbg_blockstatewrapper_free: (a: number, b: number) => void;
export const __wbg_lazychunkiterator_free: (a: number, b: number) => void;
export const __wbg_schematicbuilderwrapper_free: (a: number, b: number) => void;
export const __wbg_schematicwrapper_free: (a: number, b: number) => void;
export const blockstatewrapper_name: (a: number) => [number, number];
export const blockstatewrapper_new: (a: number, b: number) => number;
export const blockstatewrapper_properties: (a: number) => any;
export const blockstatewrapper_with_property: (a: number, b: number, c: number, d: number, e: number) => void;
export const debug_json_schematic: (a: number) => [number, number];
export const debug_schematic: (a: number) => [number, number];
export const lazychunkiterator_current_position: (a: number) => number;
export const lazychunkiterator_has_next: (a: number) => number;
export const lazychunkiterator_next: (a: number) => any;
export const lazychunkiterator_reset: (a: number) => void;
export const lazychunkiterator_skip_to: (a: number, b: number) => void;
export const lazychunkiterator_total_chunks: (a: number) => number;
export const schematicbuilderwrapper_build: (a: number) => [number, number, number];
export const schematicbuilderwrapper_fromTemplate: (a: number, b: number) => [number, number, number];
export const schematicbuilderwrapper_map: (a: number, b: number, c: number, d: number) => number;
export const schematicbuilderwrapper_name: (a: number, b: number, c: number) => number;
export const schematicbuilderwrapper_new: () => number;
export const schematicwrapper_blocks: (a: number) => any;
export const schematicwrapper_blocks_indices: (a: number) => any;
export const schematicwrapper_chunks: (a: number, b: number, c: number, d: number) => any;
export const schematicwrapper_chunks_indices: (a: number, b: number, c: number, d: number) => any;
export const schematicwrapper_chunks_indices_with_strategy: (a: number, b: number, c: number, d: number, e: number, f: number, g: number, h: number, i: number) => any;
export const schematicwrapper_chunks_with_strategy: (a: number, b: number, c: number, d: number, e: number, f: number, g: number, h: number, i: number) => any;
export const schematicwrapper_compileInsign: (a: number) => [number, number, number];
export const schematicwrapper_copy_region: (a: number, b: number, c: number, d: number, e: number, f: number, g: number, h: number, i: number, j: number, k: number, l: any) => [number, number];
export const schematicwrapper_create_lazy_chunk_iterator: (a: number, b: number, c: number, d: number, e: number, f: number, g: number, h: number, i: number) => number;
export const schematicwrapper_debug_info: (a: number) => [number, number];
export const schematicwrapper_extractSigns: (a: number) => any;
export const schematicwrapper_flip_region_x: (a: number, b: number, c: number) => [number, number];
export const schematicwrapper_flip_region_y: (a: number, b: number, c: number) => [number, number];
export const schematicwrapper_flip_region_z: (a: number, b: number, c: number) => [number, number];
export const schematicwrapper_flip_x: (a: number) => void;
export const schematicwrapper_flip_y: (a: number) => void;
export const schematicwrapper_flip_z: (a: number) => void;
export const schematicwrapper_from_data: (a: number, b: number, c: number) => [number, number];
export const schematicwrapper_from_litematic: (a: number, b: number, c: number) => [number, number];
export const schematicwrapper_from_schematic: (a: number, b: number, c: number) => [number, number];
export const schematicwrapper_getChunkData: (a: number, b: number, c: number, d: number, e: number, f: number, g: number) => any;
export const schematicwrapper_get_all_block_entities: (a: number) => any;
export const schematicwrapper_get_all_palettes: (a: number) => any;
export const schematicwrapper_get_allocated_dimensions: (a: number) => [number, number];
export const schematicwrapper_get_available_schematic_versions: (a: number) => any;
export const schematicwrapper_get_block: (a: number, b: number, c: number, d: number) => [number, number];
export const schematicwrapper_get_block_count: (a: number) => number;
export const schematicwrapper_get_block_entity: (a: number, b: number, c: number, d: number) => any;
export const schematicwrapper_get_block_string: (a: number, b: number, c: number, d: number) => [number, number];
export const schematicwrapper_get_block_with_properties: (a: number, b: number, c: number, d: number) => number;
export const schematicwrapper_get_bounding_box: (a: number) => any;
export const schematicwrapper_get_chunk_blocks: (a: number, b: number, c: number, d: number, e: number, f: number, g: number) => any;
export const schematicwrapper_get_chunk_blocks_indices: (a: number, b: number, c: number, d: number, e: number, f: number, g: number) => any;
export const schematicwrapper_get_default_region_palette: (a: number) => any;
export const schematicwrapper_get_dimensions: (a: number) => [number, number];
export const schematicwrapper_get_optimization_info: (a: number) => any;
export const schematicwrapper_get_palette: (a: number) => any;
export const schematicwrapper_get_palette_from_region: (a: number, b: number, c: number) => any;
export const schematicwrapper_get_region_bounding_box: (a: number, b: number, c: number) => any;
export const schematicwrapper_get_region_names: (a: number) => [number, number];
export const schematicwrapper_get_tight_bounds_max: (a: number) => [number, number];
export const schematicwrapper_get_tight_bounds_min: (a: number) => [number, number];
export const schematicwrapper_get_tight_dimensions: (a: number) => [number, number];
export const schematicwrapper_get_volume: (a: number) => number;
export const schematicwrapper_new: () => number;
export const schematicwrapper_print_schematic: (a: number) => [number, number];
export const schematicwrapper_rotate_region_x: (a: number, b: number, c: number, d: number) => [number, number];
export const schematicwrapper_rotate_region_y: (a: number, b: number, c: number, d: number) => [number, number];
export const schematicwrapper_rotate_region_z: (a: number, b: number, c: number, d: number) => [number, number];
export const schematicwrapper_rotate_x: (a: number, b: number) => void;
export const schematicwrapper_rotate_y: (a: number, b: number) => void;
export const schematicwrapper_rotate_z: (a: number, b: number) => void;
export const schematicwrapper_setBlockWithNbt: (a: number, b: number, c: number, d: number, e: number, f: number, g: any) => [number, number];
export const schematicwrapper_set_block: (a: number, b: number, c: number, d: number, e: number, f: number) => void;
export const schematicwrapper_set_block_with_properties: (a: number, b: number, c: number, d: number, e: number, f: number, g: any) => [number, number];
export const schematicwrapper_to_litematic: (a: number) => [number, number, number, number];
export const schematicwrapper_to_schematic: (a: number) => [number, number, number, number];
export const schematicwrapper_to_schematic_version: (a: number, b: number, c: number) => [number, number, number, number];
export const start: () => void;
export const __wbindgen_malloc: (a: number, b: number) => number;
export const __wbindgen_realloc: (a: number, b: number, c: number, d: number) => number;
export const __wbindgen_free: (a: number, b: number, c: number) => void;
export const __wbindgen_exn_store: (a: number) => void;
export const __externref_table_alloc: () => number;
export const __wbindgen_externrefs: WebAssembly.Table;
export const __externref_table_dealloc: (a: number) => void;
export const __externref_drop_slice: (a: number, b: number) => void;
export const __wbindgen_start: () => void;
